SystemOrganization addCategory: #MonticelloRemoteRepositories!

Object subclass: #MCServerCredentials
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCServerCredentials commentStamp: '<historical>' prior: 0!
I store a username and password.
!

!MCServerCredentials class methodsFor: 'instance creation' stamp: 'CamilloBruni 4/11/2013 17:33'!
user: nameString password: passwordString

	^ self new
			username: nameString;
			password: passwordString;
			yourself.! !

!MCServerCredentials methodsFor: 'accessing' stamp: 'SeanDeNigris 8/26/2012 18:33'!
password

	^ password.! !

!MCServerCredentials methodsFor: 'private' stamp: 'SeanDeNigris 8/26/2012 18:25'!
password: aString 

	password := aString.! !

!MCServerCredentials methodsFor: 'accessing' stamp: 'SeanDeNigris 8/26/2012 18:33'!
username

	^ username.! !

!MCServerCredentials methodsFor: 'private' stamp: 'SeanDeNigris 8/26/2012 18:25'!
username: aString 

	username := aString.! !

Object subclass: #MCServerRegistry
	instanceVariableNames: 'registry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

MCServerRegistry class
	instanceVariableNames: 'uniqueInstance'!

!MCServerRegistry commentStamp: 'TorstenBergmann 2/5/2014 13:56' prior: 0!
Registry for metacello servers!

MCServerRegistry class
	instanceVariableNames: 'uniqueInstance'!

!MCServerRegistry class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 11:11'!
uniqueInstance

	^ uniqueInstance ifNil: [ uniqueInstance := self new ].! !

!MCServerRegistry methodsFor: 'initialization' stamp: 'SeanDeNigris 8/26/2012 18:18'!
initialize

	super initialize.
	registry := Dictionary new.! !

!MCServerRegistry methodsFor: 'public' stamp: 'CamilloBruni 4/11/2013 17:33'!
on: repositoryUrl beUser: nameString withPassword: passwordString

	| credentials |
	credentials := MCServerCredentials user: nameString password: passwordString.
	registry at: repositoryUrl put: credentials.! !

!MCServerRegistry methodsFor: 'private' stamp: 'CamilloBruni 2/19/2013 18:30'!
repositoryAt: urlString credentialsDo: aBlock


	| possibleMatches bestMatch |
	possibleMatches := registry associations select: [ :e | urlString beginsWith: e key ].
	possibleMatches isEmpty ifTrue: [ ^ aBlock value: '' value: '' ].
	bestMatch := possibleMatches inject: possibleMatches anyOne into: [ :last :new | 
		((new key asUrl path size > last key asUrl path size) 
		"this is ugly, but URL always returns a path element, even an empty one..."
		or: [ last key asUrl path size = 1 and: [ last key asUrl path last isEmpty ]]) 
			ifTrue: [ new ]
			ifFalse: [ last ]].
	^ aBlock value: bestMatch value username value: bestMatch value password.! !

MCFileBasedRepository subclass: #MCFtpRepository
	instanceVariableNames: 'host directory user password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCFtpRepository commentStamp: '<historical>' prior: 0!
I am an monticello repository implementation for the FTP protocol.!

!MCFtpRepository class methodsFor: 'instance creation' stamp: 'ChristopheDemarey 7/7/2015 10:49'!
basicFromUrl: aZnUrl
	^ self
		host: (aZnUrl hasPort ifTrue: [ aZnUrl host, ':', aZnUrl port asString ] ifFalse: [ aZnUrl host ])
		directory: aZnUrl path "MCFtpRepository assumes NO prefixed / in the path"
		user: (aZnUrl username ifNil: [ '' ])
		password: (aZnUrl password ifNil: [ '' ])! !

!MCFtpRepository class methodsFor: 'accessing' stamp: 'avi 9/16/2003 13:57'!
creationTemplate
	^
'MCFtpRepository
	host: ''modules.squeakfoundation.org''
	directory: ''mc''
	user: ''squeak''
	password: ''squeak'''
	! !

!MCFtpRepository class methodsFor: 'accessing' stamp: 'avi 9/16/2003 13:57'!
description
	^ 'FTP'! !

!MCFtpRepository class methodsFor: 'instance creation' stamp: 'avi 9/16/2003 13:57'!
host: host directory: directory user: user password: password
	^ self new
		host: host;
		directory: directory;
		user: user;
		password: password! !

!MCFtpRepository class methodsFor: 'accessing' stamp: 'bkv 2/18/2004 20:38'!
templateCreationSelector
	^ #host:directory:user:password: ! !

!MCFtpRepository class methodsFor: 'accessing' stamp: 'ChristopheDemarey 7/7/2015 10:25'!
urlSchemes
	^ #(ftp)! !

!MCFtpRepository methodsFor: 'enumerating' stamp: 'stephaneducasse 2/4/2006 20:47'!
clientDo: aBlock
	| client |
	client := FTPClient openOnHostNamed: host.
	client loginUser: user password: password.
	directory isEmpty ifFalse: [client changeDirectoryTo: directory].
	^ [aBlock value: client] ensure: [client close]! !

!MCFtpRepository methodsFor: 'required' stamp: 'MarcusDenker 9/24/2014 15:34'!
description
	^ 'ftp://', host, '/', directory! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
directory: dirPath
	directory := dirPath! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
host: hostname
	host := hostname! !

!MCFtpRepository methodsFor: 'interface' stamp: 'CamilloBruni 3/2/2012 13:02'!
loadAllFileNames
	^ self clientDo:
		[:client |
		self parseDirectoryListing: client getDirectory]! !

!MCFtpRepository methodsFor: 'parsing' stamp: 'stephaneducasse 2/4/2006 20:47'!
parseDirectoryListing: aString
	| stream files line tokens |
	stream := aString readStream.
	files := OrderedCollection new.
	[stream atEnd] whileFalse:
		[line := stream nextLine.
		tokens := line findTokens: ' '.
		tokens size > 2 ifTrue: [files add: tokens last]].
	^ files! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
password: passwordString
	password := passwordString! !

!MCFtpRepository methodsFor: 'required' stamp: 'nice 1/5/2010 15:59'!
readStreamForFileNamed: aString do: aBlock
	
	^ self clientDo:
		[:client | | stream |
		client binary.
		stream := RWBinaryOrTextStream on: String new.
		stream nextPutAll: (client getFileNamed: aString).
		aBlock value: stream reset]! !

!MCFtpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
user: userString
	user := userString! !

!MCFtpRepository methodsFor: 'required' stamp: 'stephaneducasse 2/4/2006 20:47'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	self clientDo:
		[:client |
		client binary.
		client putFileStreamContents: stream reset as: aString]! !

MCFileBasedRepository subclass: #MCHttpRepository
	instanceVariableNames: 'location user password readerCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCHttpRepository commentStamp: '<historical>' prior: 0!
I am general http repository for monticello.
I support the general protocol for listing files in a remote repository.!

MCHttpRepository subclass: #MCGemstoneRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCGemstoneRepository commentStamp: 'CamilloBruni 2/8/2012 18:10' prior: 0!
An explicit subclass of MCHttpRepository to provide a template for gemstone repositories.!

!MCGemstoneRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 6/26/2013 13:37'!
baseURL
	^ 'http://ss3.gemtalksystems.com/ss/'! !

!MCGemstoneRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 2/8/2012 18:12'!
description
	^ 'ss3.gemstone.com'! !

!MCGemstoneRepository class methodsFor: 'testing' stamp: 'CamilloBruni 6/26/2013 13:37'!
isResponsibleFor: aUrl
	^ (aUrl includesSubstring: 'ss3.gemstone.com/') or: [  aUrl includesSubstring: 'ss3.gemtalksystems.com/' ]! !

!MCGemstoneRepository methodsFor: 'interface' stamp: 'CamilloBruni 4/23/2012 20:22'!
includesVersionNamed: aString
	"directly do a filename check since squeaksource only stores mcz"
	^ self includesFileNamed: aString, '.mcz'! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 10/21/2012 13:25'!
baseURL
	^ ''! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'ChristopheDemarey 7/7/2015 12:33'!
basicFromUrl: aZnUrl
	^ aZnUrl hasUsername
		ifFalse: [ self location: aZnUrl location ]
		ifTrue: [ (self repositoryClassFor: aZnUrl location) new
				location: aZnUrl location;
				user: aZnUrl username;
				password: aZnUrl password;
				yourself  ]! !

!MCHttpRepository class methodsFor: 'initialization' stamp: 'bf 7/28/2005 19:44'!
clearPasswords
	self allSubInstancesDo: [:ea | ea password: ''].
! !

!MCHttpRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 10/21/2012 13:24'!
creationTemplate
	^self creationTemplateLocation: self baseURL
		user: ''
		password: ''
! !

!MCHttpRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 2/15/2012 15:21'!
creationTemplateLocation: location user: user password: password
	^
self name, '
	location: {1}
	user: {2}
	password: {3}' format: {location printString. user printString. password printString}! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'ab 7/24/2003 21:20'!
description
	^ 'HTTP'! !

!MCHttpRepository class methodsFor: 'testing' stamp: 'CamilloBruni 4/6/2013 23:04'!
isResponsibleFor: aURLString
	"Override in subclasses to enable custom instances for certain URLs"
	^ true! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 5/7/2013 14:34'!
location: location
	^ MCServerRegistry uniqueInstance 
		repositoryAt: location credentialsDo: [ :username :password |
			(self repositoryClassFor: location) new
				location: location;
				user: username;
				password: password;
				yourself ]! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'SeanDeNigris 8/26/2012 18:22'!
location: aUrl user: user password: password

	| result |
	result := self location: aUrl.
	user ifNotEmpty: [
		result
			user: user;
			password: password ].
	^ result.! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'MarcusDenker 4/16/2015 15:48'!
pharoInboxRepository
	^ self location: 'http://smalltalkhub.com/mc/Pharo/Pharo50Inbox/main'.! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'MarcusDenker 4/16/2015 15:48'!
pharoRepository
	^ self location: 'http://smalltalkhub.com/mc/Pharo/Pharo50/main'.! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 10/21/2012 13:26'!
project: aProjectIdentifier
	^ self location: self baseURL, aProjectIdentifier! !

!MCHttpRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 4/6/2013 23:33'!
repositoryClassFor: location
	MCHttpRepository subclassesDo: [ :subclass | 
		(subclass isResponsibleFor: location)
			ifTrue: [ ^ subclass ]].
	^ MCHttpRepository! !

!MCHttpRepository class methodsFor: 'accessing' stamp: 'ChristopheDemarey 7/7/2015 09:59'!
urlSchemes
	^ #(http https)! !

!MCHttpRepository methodsFor: 'converting' stamp: 'CamilloBruni 9/14/2012 17:24'!
asCreationTemplate
	^self class creationTemplateLocation: self location user: user password: password! !

!MCHttpRepository methodsFor: 'private' stamp: 'SeanDeNigris 8/27/2012 12:09'!
assertBinaryResponse: response 

	response contentType isBinary ifFalse: [ 
		MCRepositoryError signal: 'Expected a binary response instead of ', response contentType printString ].! !

!MCHttpRepository methodsFor: 'private' stamp: 'SeanDeNigris 8/27/2012 12:05'!
assertNonBinaryResponse: response 
	
	response contentType isBinary ifTrue: [ MCRepositoryError signal: 'Did not expect a binary response but got ', response contentType printString ].! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'SeanDeNigris 7/17/2012 15:50'!
cacheRawVersionNamed: aString stream: contents
	"directly forward the contents to the cache repository. 
	this avoids and unnecessary serialization step"
	MCCacheRepository uniqueInstance 
		writeStreamForFileNamed: aString 
		replace: true 
		do: [ :s| s nextPutAll: contents ]! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:42'!
credentials
	^ MCServerCredentials user: self user password: self password! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:45'!
credentials: mcServerCredentials
	self user: mcServerCredentials username.
	self password: mcServerCredentials password.! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:24'!
description
	^ self location! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'SeanDeNigris 6/20/2012 23:28'!
displayProgress: label during: workBlock
	| nextUpdateTime |
	nextUpdateTime := 0.
	^UIManager default displayProgress: label 
		from: 0.0 to: 1.0 during:[:bar|
			[workBlock value] on: HTTPProgress do:[:ex|
				(ex total == nil or: [ex amount == nil]) ifFalse:[
					(nextUpdateTime < Time millisecondClockValue 
						or:[ex total = ex amount]) ifTrue:[
							bar current: ex amount asFloat / ex total asFloat.
							nextUpdateTime := Time millisecondClockValue + 100.
					].
				].
				ex resume.
			]
		].
! !

!MCHttpRepository methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/5/2012 21:54'!
entityStreamContents: aBlock
	"Generate output in a buffer because we need the length"
	
	| stream |
	stream := RWBinaryOrTextStream on: String new.
	aBlock value: stream.
	stream reset.
	^ (ZnStreamingEntity type: ZnMimeType applicationOctetStream)
		stream: stream;
		contentLength: stream size;
		yourself! !

!MCHttpRepository methodsFor: 'actions' stamp: 'al 12/12/2005 11:06'!
flushCache
	super flushCache.
	readerCache := nil.! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'CamilloBruni 4/11/2013 17:22'!
handleUnsuccessfulResponse: aZnResponse
	(#(403 401) includes: aZnResponse code)
		ifTrue: [ MCPermissionDenied signalFor: self ].
	Error signal: 'Could not save version.'! !

!MCHttpRepository methodsFor: 'private' stamp: 'CamilloBruni 9/18/2013 14:17'!
httpClient
	"Return a new, specifically configured instance of the HTTP client for internal use.
	Note how we request GZIP compression and will signal progress."

	^ ZnClient new
		systemPolicy;
		beOneShot;
		username: self user password: self password;
	
		signalProgress: true;
		yourself! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'CamilloBruni 6/15/2013 10:13'!
loadAllFileNames
	| client |
	self displayProgress: 'Loading all file names from ', self description during: [
		client := self httpClient.
		client
			ifFail: [ :exception | 
				(exception className beginsWith: 'Zn')
					ifTrue: [ MCRepositoryError signal: 'Could not access ', self location, ': ', exception printString ]
					ifFalse: [ exception pass ] ];
			url: self locationWithTrailingSlash;
			queryAt: 'C' put: 'M;O=D';	"legacy that some servers maybe expect"
			get.
		self assertNonBinaryResponse: client response ].
	^ self parseFileNamesFromStream: client contents readStream! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:24'!
location
	^ location! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
location: aUrlString
	location := aUrlString! !

!MCHttpRepository methodsFor: 'actions' stamp: 'CamilloBruni 9/14/2012 17:24'!
locationWithTrailingSlash
	^ (self location endsWith: '/')
		ifTrue: [self location]
		ifFalse: [self location, '/']! !

!MCHttpRepository methodsFor: 'actions' stamp: 'SvenVanCaekenberghe 10/27/2013 12:19'!
parseFileNamesFromStream: aStream
	| names fullName |
	names := OrderedCollection new.
	[aStream atEnd] whileFalse:
		[[aStream upTo: $<. {$a. $A. nil} includes: aStream next] whileFalse.
		aStream upTo: $".
		aStream atEnd ifFalse: [
			fullName := aStream upTo: $".
			names add: fullName urlDecoded ]].
	^ names! !

!MCHttpRepository methodsFor: 'actions' stamp: 'SD 8/14/2015 05:54'!
password
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^pwd].

	self user isEmpty ifTrue: [^password ifNil: ['']].

	password isEmptyOrNil ifTrue: [
		| answer |
		user isEmptyOrNil ifTrue: [
			"Give the user a chance to change the login"
			answer := UIManager default request: 'User name for ', String cr, location
				initialAnswer: ''.
			answer isEmptyOrNil
				ifTrue: [^password]
				ifFalse: [self user: answer] ].
		
		password := UIManager default requestPassword: 'Password for "', self user, '" at ', String cr, self location.
		"The user doesn't care about logging into this repo. Clear the username so they will not be prompted every time"
		password isEmptyOrNil ifTrue: [ user := '' ].
	].

	^ password! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
password: passwordString
	password := passwordString! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/18/2012 18:35'!
project
	"Return a project name"
	^ (self location splitOn: $/) last! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'EstebanLorenzano 4/25/2013 10:50'!
readStreamForFileNamed: aString do: aBlock
	| client |
	self displayProgress: 'Downloading ', aString during: [
		client := self httpClient.
		client
			ifFail: [ :exception | self error: 'Could not load ', aString, ': ', exception printString ];
			get: (self urlForFileNamed: aString).
		self assertBinaryResponse: client response.
		"immediately cache the version and avoid an unnecessary serialization"
		self cacheRawVersionNamed: aString stream: client contents ].
	^ aBlock value: client contents readStream! !

!MCHttpRepository methodsFor: 'actions' stamp: 'NicolaiHess 8/27/2015 10:33'!
retryOnCredentialRequest: aBlock
	aBlock	on: MCPermissionDenied do: [ :error | |credentials| 
		credentials := MCCredentialsRequest 
							signalUrl: self location
							username: self user
							password: self password.
		credentials 
			ifNotNil: [ 
				self credentials: credentials.
				^ self retryOnCredentialRequest: aBlock ]]! !

!MCHttpRepository methodsFor: 'storing' stamp: 'NicolaiHess 8/27/2015 10:38'!
storeVersion: aVersion
	self retryOnCredentialRequest: [ super storeVersion: aVersion ]! !

!MCHttpRepository methodsFor: 'actions' stamp: 'SvenVanCaekenberghe 10/27/2013 11:46'!
urlForFileNamed: aString
	^ self locationWithTrailingSlash, aString urlEncoded! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'SD 8/14/2015 06:05'!
user
	self userAndPasswordFromSettingsDo: [:usr :pwd | ^usr].
	"not in settings"
	user isEmptyOrNil ifFalse: [ ^user ].
	^ ''! !

!MCHttpRepository methodsFor: 'accessing' stamp: 'stephaneducasse 2/4/2006 20:47'!
user: userString
	user := userString! !

!MCHttpRepository methodsFor: 'actions' stamp: 'CamilloBruni 9/14/2012 17:25'!
userAndPasswordFromSettingsDo: aBlock
	"The mcSettings file in ExternalSettings preferenceDirectory should contain entries for each account:
	
		account1: *myhost.mydomain* user:password
		account2: *otherhost.mydomain/somerep* dXNlcjpwYXNzd29yZA==

	That is it must start with 'account', followed by anything to distinguish accounts, and a colon. Then comes a match expression for the repository url, and after a space the user:password string.
	
	To not have the clear text password on your disc, you can base64 encode it:
			(Base64MimeConverter mimeEncode: 'user:password' readStream) contents
	"

	
	Settings ifNotNil: [
		Settings keysAndValuesDo: [:key :value | | entry userAndPassword |
			(key asLowercase beginsWith: 'account') ifTrue: [
				entry := value findTokens: '	 '.
				(entry first match: self location) ifTrue: [
					userAndPassword := entry second.
					(userAndPassword includes: $:) ifFalse: [
						userAndPassword := (Base64MimeConverter mimeDecodeToChars: userAndPassword readStream) contents].
					userAndPassword := userAndPassword findTokens: $:.
					^aBlock value: userAndPassword first 
						value: userAndPassword second 
					]
			]
		]
	].
	^nil! !

!MCHttpRepository methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
versionReaderForFileNamed: aString
	readerCache ifNil: [readerCache := Dictionary new].
	^ readerCache at: aString ifAbsent:
		[self resizeCache: readerCache.
		super versionReaderForFileNamed: aString do:
			[:r |
			r ifNotNil: [readerCache at: aString put: r]]]
	! !

!MCHttpRepository methodsFor: 'actions' stamp: 'marcus.denker 11/10/2008 10:04'!
versionReaderForFileNamed: aString do: aBlock
	^ (self versionReaderForFileNamed: aString) ifNotNil: aBlock! !

!MCHttpRepository methodsFor: 'i/o' stamp: 'CamilloBruni 4/11/2013 17:19'!
writeStreamForFileNamed: aString replace: ignoreBoolean do: aBlock
	| entity |
	entity := self entityStreamContents: aBlock.
	self displayProgress: 'Uploading ', aString during: [
		self httpClient
			entity: entity;
			ifFail: [ :exception | 
				(exception isKindOf: ZnHttpUnsuccessful) 
					ifTrue: [ ^ self handleUnsuccessfulResponse: exception response ].
				self error: 'Could not save ', aString, ': ', exception printString ];
			url: (self urlForFileNamed: aString);
			put ]! !

MCHttpRepository subclass: #MCSmalltalkhubRepository
	instanceVariableNames: 'owner project'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCSmalltalkhubRepository commentStamp: '<historical>' prior: 0!
I am specialized version of an MCHttpRepository for http://smalltalkhub.com.
I support a faster mcz listing that does not rely on parsing an html size.!

!MCSmalltalkhubRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2013 14:31'!
baseURL
	^ self smalltalkhubUrl, 'mc/'! !

!MCSmalltalkhubRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 9/14/2012 17:15'!
creationTemplate
	^self creationTemplateOwner: ''
		project: ''
		user: ''
		password: ''
! !

!MCSmalltalkhubRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 9/14/2012 17:23'!
creationTemplateOwner: owner project: project user: user password: password
	^ String streamContents: [ :s|
		s 
			nextPutAll: self name; cr;
			tab; nextPutAll: 'owner: '; print: owner; cr;
			tab; nextPutAll: 'project: '; print: project; cr;
			tab; nextPutAll: 'user: '; print: user; cr;
			tab; nextPutAll: 'password: '; print: password ].! !

!MCSmalltalkhubRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 16:38'!
description
	^ 'smalltalkhub.com'! !

!MCSmalltalkhubRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2013 14:31'!
hostName
	^ 'smalltalkhub.com'! !

!MCSmalltalkhubRepository class methodsFor: 'testing' stamp: 'CamilloBruni 5/7/2013 14:31'!
isResponsibleFor: aUrl
	^ aUrl includesSubstring: self hostName! !

!MCSmalltalkhubRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 10/9/2012 14:54'!
owner: owner project: project 
	^ self
		owner: owner
		project: project
		user: String empty
		password: String empty! !

!MCSmalltalkhubRepository class methodsFor: 'instance creation' stamp: 'CamilloBruni 9/14/2012 17:22'!
owner: owner project: project user: user password: password
	^ self new
		owner: owner;
		project: project;
		user: user;
		password: password;
		yourself! !

!MCSmalltalkhubRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 5/7/2013 14:30'!
smalltalkhubUrl
	^ 'http://', self hostName, '/'! !

!MCSmalltalkhubRepository methodsFor: 'converting' stamp: 'CamilloBruni 9/14/2012 17:20'!
asCreationTemplate
	^self class 
		creationTemplateOwner: self owner project: self project user: user password: password! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'SvenVanCaekebenberghe 2/19/2016 15:24'!
includesFileNamed: aString
	"avoid the slower default method and simply do a head request "
	self httpClient
		numberOfRetries: 0;
		ifFail: [ :exception | 
			((exception isKindOf: ZnHttpUnsuccessful) and: [ exception response isNotFound ])
				ifTrue: [ ^ false ].
			exception pass];
		head: (self urlForFileNamed: aString).
	^ true! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'CamilloBruni 9/14/2012 16:37'!
includesVersionNamed: aString
	"directly do a filename check since squeaksource only stores mcz"
	^ self includesFileNamed: aString, '.mcz'! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'CamilloBruni 6/15/2013 10:14'!
loadAllFileNames
	| client |
	client := self httpClient.
	client
		ifFail: [ :exception | self error: 'Could not access ', self location, ': ', exception printString ];
		url: self locationWithTrailingSlash;
		queryAt: 'format' put: 'raw';
		get.
	self assertNonBinaryResponse: client response.
	^ self parseFileNamesFromStream: client contents! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:25'!
location
	^ 'http://smalltalkhub.com/mc/', self owner, '/', self project, '/main/' ! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 4/6/2013 23:12'!
location: aUrlString
	| pathSegments |
	(self class isResponsibleFor: aUrlString)
		ifFalse: [ Error signal: 'Please provide a smalltalkhub url' ].
	"extract the smalltalkhub properties from the path part in the given URL"
	pathSegments := aUrlString asZnUrl pathSegments reject: [ :each | each = 'mc' ].
	self owner: pathSegments first.
	self project: pathSegments second.! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:24'!
locationWithTrailingSlash
	^ self location! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:21'!
owner
	
	^ owner! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/18/2012 18:36'!
owner: aString
	
	owner := aString! !

!MCSmalltalkhubRepository methodsFor: 'interface' stamp: 'BernardoContreras 3/8/2015 19:35'!
parseFileNamesFromStream: aNewLineDelimitedString
	^ aNewLineDelimitedString 
		ifNil: [ ^ OrderedCollection new ]
		ifNotNil: [ aNewLineDelimitedString substrings: String crlf ]! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/14/2012 17:21'!
project
	
	^ project! !

!MCSmalltalkhubRepository methodsFor: 'accessing' stamp: 'CamilloBruni 9/18/2012 18:36'!
project: aString
	
	project := aString! !

MCHttpRepository subclass: #MCSqueaksourceRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCSqueaksourceRepository commentStamp: '<historical>' prior: 0!
I am a specialized MCHttpRepository for http://squeaksource.com.
I optimize checks for existing files by doing a head request instead of reloading the whole file list first.!

!MCSqueaksourceRepository class methodsFor: 'creation template' stamp: 'CamilloBruni 10/21/2012 13:25'!
baseURL
	^ 'http://squeaksource.com/'! !

!MCSqueaksourceRepository class methodsFor: 'accessing' stamp: 'CamilloBruni 2/8/2012 18:13'!
description
	^ 'squeaksource.com'! !

!MCSqueaksourceRepository class methodsFor: 'testing' stamp: 'CamilloBruni 4/6/2013 23:07'!
isResponsibleFor: aUrl
	^ aUrl includesSubstring: 'squeaksource.com/'! !

!MCSqueaksourceRepository methodsFor: 'interface' stamp: 'SvenVanCaekebenberghe 2/19/2016 15:24'!
includesFileNamed: aString
	
	"avoid the slower default method and simply do a head request "
	self httpClient
		numberOfRetries: 0;
		ifFail: [ :exception | 
			((exception isKindOf: ZnHttpUnsuccessful) and: [ exception response isNotFound ])
				ifTrue: [ ^ false ].
			exception pass];
		head: (self urlForFileNamed: aString).
		
	^ true! !

!MCSqueaksourceRepository methodsFor: 'interface' stamp: 'CamilloBruni 4/23/2012 20:22'!
includesVersionNamed: aString
	"directly do a filename check since squeaksource only stores mcz"
	^ self includesFileNamed: aString, '.mcz'! !

Error subclass: #MCPermissionDenied
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCPermissionDenied commentStamp: '<historical>' prior: 0!
I am signalled when a repositroy cannot save or access a version.!

!MCPermissionDenied class methodsFor: 'signalling' stamp: 'CamilloBruni 4/11/2013 17:17'!
signalFor: aRepository
	^ self new
		repository: aRepository;
		signal! !

!MCPermissionDenied methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:17'!
repository
	^ repository! !

!MCPermissionDenied methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:17'!
repository: anObject
	repository := anObject! !

Notification subclass: #MCCredentialsRequest
	instanceVariableNames: 'username password url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloRemoteRepositories'!

!MCCredentialsRequest commentStamp: '<historical>' prior: 0!
I am used to prompt in a UI indepentent way for a username and a password.
If the user successfully entered the two strings I return an MCServerCredential if not I return nil.!

!MCCredentialsRequest class methodsFor: 'signalling' stamp: 'CamilloBruni 4/11/2013 17:35'!
signalUrl: aUrl
	^ self new 
		url: aUrl;
		signal! !

!MCCredentialsRequest class methodsFor: 'signalling' stamp: 'CamilloBruni 4/11/2013 17:49'!
signalUrl: aUrl username: username password: password
	^ self new 
		url: aUrl;
		username: username;
		password: password;
		signal! !

!MCCredentialsRequest methodsFor: 'exceptiondescription' stamp: 'CamilloBruni 4/11/2013 17:49'!
defaultAction
	
	username := UIManager default
			request: self userMessagePrompt
			initialAnswer: self username
			title: 'User' translated.
	(username isNil or: [ username isEmpty ])
		ifTrue: [ ^ nil ].
	password := UIManager default
			requestPassword: self passwordMessagePrompt.
	^ MCServerCredentials user: username password: password! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:48'!
password
	^ password! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:48'!
password: anObject
	password := anObject! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:36'!
passwordMessagePrompt
	^ 'Please enter a password for ', self url asString! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:35'!
url
	^ url! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:35'!
url: anObject
	url := anObject! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'NicolaiHess 8/24/2015 00:19'!
userMessagePrompt
self haltOnce.
	^ 'Please enter a username for ', self url asString! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:48'!
username
	^ username! !

!MCCredentialsRequest methodsFor: 'accessing' stamp: 'CamilloBruni 4/11/2013 17:48'!
username: anObject
	username := anObject! !
